use std::io::{self, stdin, IsTerminal};

#[cfg(not(target_arch = "wasm32"))]
mod cli;

#[cfg(not(target_arch = "wasm32"))]
use cli::get_input;
use georgeemu::memory::{Mem, MemHandle};
#[cfg(not(target_arch = "wasm32"))]
use georgeemu::GeorgeEmu;
#[cfg(not(target_arch = "wasm32"))]
use minifb::{Scale, ScaleMode, Window, WindowOptions};

use std::{fs::File, io::Read};

#[cfg(not(target_arch = "wasm32"))]
fn main() -> io::Result<()> {
    use std::io::{stdout, Write};

    use georgeemu::cpu::Cpu;

    let mut input = stdin();

    if !input.is_terminal() {
        let mut bytes = [0u8; 0x8000];
        input.read_exact(&mut bytes).unwrap();
        let memory = make_memory(&bytes);
        let mut cpu = Cpu::new(memory);
        let final_output = cpu.run_to_end();

        let _ = stdout().write_all(&final_output);
        Ok(())
    } else {
        let window = Window::new(
            "ʕ·ᴥ·ʔ-☆",
            512,
            380,
            WindowOptions {
                resize: true,
                borderless: true,
                title: true,
                transparency: false,
                scale: Scale::FitScreen,
                scale_mode: ScaleMode::AspectRatioStretch,
                topmost: false,
                none: true,
            },
        )
        .unwrap();

        let config = get_input();
        let rom = match config.rom {
            Some(path) => {
                let mut file = File::open(path).unwrap();
                let mut bin = vec![0; 0x8000];
                file.read_exact(&mut bin).unwrap();
                bin.try_into().unwrap()
            }
            None => *include_bytes!(concat!(env!("CARGO_MANIFEST_DIR"), "/roms/george.rom")),
        };
        let mut emu = GeorgeEmu::builder().rom(rom).window(window).build();

        emu.run();
        Ok(())
    }
}

#[cfg(not(target_arch = "wasm32"))]
fn make_memory(bytes: &[u8]) -> MemHandle {
    let mut memory = Mem::new();
    memory.load_bytes(bytes);
    MemHandle::new(memory)
}

#[cfg(target_arch = "wasm32")]
fn main() {}
