#[cfg(not(target_arch = "wasm32"))]
use std::io::{self, Write};
use std::{env, fs::File, io::Read, path::Path};

#[cfg(not(target_arch = "wasm32"))]
use termion::{
    color::{self, Bg, Fg},
    cursor::Goto,
};

use crate::{
    cpu::CpuController,
    memory::{MemHandle, MemoryReader},
};
use minifb::Window;

#[cfg(not(target_arch = "wasm32"))]
const FG_COLOR: u32 = 0xFFCC00;
#[cfg(not(target_arch = "wasm32"))]
const BG_COLOR: u32 = 0x110500;

// Wasm colors are ABGR
#[cfg(target_arch = "wasm32")]
const FG_COLOR: u32 = 0xFF00CCFF;
#[cfg(target_arch = "wasm32")]
const BG_COLOR: u32 = 0xFF000511;

const WIDTH: usize = 512;
const HEIGHT: usize = 380;

// pub fn get_char_bin<P>(char_rom: Option<P>) -> [u8; 0x8000]
// where
//     P: AsRef<Path>,
// {
//     match char_rom {
//         Some(path) => {
//             let mut file = File::open(path).unwrap();
//             let mut bin = vec![0; 0x8000];
//             file.read_exact(&mut bin).unwrap();
//             // println!("reading char rom");
//             bin.try_into().unwrap()
//         }
//         None => *include_bytes!("./roms/cozette.rom"),
//     }
// }

const CHAR_ROM: &[u8; 0x8000] = include_bytes!(concat!(env!("OUT_DIR"), "/cozette.rom"));
#[cfg(not(target_arch = "wasm32"))]
const ASCII_LOOKUP: [&str; 256] = [
    " ", "░", "▒", "▓", "♡", "♥", "⭐", "✭", "", "✦", "✨", "♀", "♂", "⚢", "⚣", "⚥", "♩", "♪",
    "♫", "♬", "ﱝ", "", "", "", "奄", "奔", "婢", "ﱜ", "ﱛ", "", "", "", " ", "!", "\"", "#",
    "$", "%", "&", "\'", "(", ")", "*", "+", ",", "-", ".", "/", "0", "1", "2", "3", "4", "5", "6",
    "7", "8", "9", ":", ";", "<", "=", ">", "?", "@", "A", "B", "C", "D", "E", "F", "G", "H", "I",
    "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z", "[", "\\",
    "]", "^", "_", "`", "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o",
    "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z", "{", "|", "}", "~", "─", "│", "┌", "└",
    "├", "┤", "┬", "┴", "┼", "╭", "╮", "╯", "╰", "╱", "╲", "╳", "═", "║", "╔", "╗", "╚", "╝", "╠",
    "╣", "╦", "╩", "╬", "", "", "", "", "", "", "", "", "", "", "", "", "", "▁", "▂",
    "▃", "▄", "▅", "▆", "▇", "█", "▉", "▊", "▋", "▌", "▍", "▎", "▏", "ʕ", "·", "ᴥ", "ʔ", "▖", "▗",
    "▘", "▙", "▚", "▛", "▜", "▝", "▞", "▟", "←", "↑", "→", "↓", "⭠", "⭡", "⭢", "⭣", "⮀", "⮁", "⮂",
    "⮃", "", "", "⏳", "", "", "", "", "", "", "", "", "", "", "", "🔮", "", "",
    "◇", "◈", "🌑", "🌒", "🌓", "🌔", "🌕", "🌖", "🌗", "🌘", "", "✔", "✘", "◆", "", "", "",
    "", "", "", "", "🎁", "", "", "", "", "⚐", "⚑", "", "", "",
];

#[derive(Debug)]
pub struct Renderer {
    memory: MemHandle,
    controller: CpuController,
}

impl Renderer {
    pub fn new(controller: CpuController, memory: MemHandle) -> Self {
        Self { memory, controller }
    }

    pub fn render(&self, window: Option<&mut Window>) {
        match window {
            Some(window) => {
                // the rest of this function is arcane wizardry
                // based on the specifics of george's weird
                // display and characters... don't fuck around w it
                let mut i = 0;
                let mut buffer = [0; WIDTH * HEIGHT];
                for char_row in 0..29 {
                    for char_col in 0..64 {
                        let ascii = self.read(0x6000 + i);
                        i += 1;
                        for row in 0..13 {
                            let byte = CHAR_ROM[ascii as usize + (row * 0x100)];
                            for bit_index in (0..8).rev() {
                                let buffer_index =
                                    ((char_row) * 13 + (row)) * 512 + (char_col * 8 + bit_index);
                                if (byte << bit_index) & 0x80 == 0x80 {
                                    buffer[buffer_index] = FG_COLOR;
                                } else {
                                    buffer[buffer_index] = BG_COLOR;
                                }
                            }
                        }
                    }
                }
                window.update_with_buffer(&buffer, WIDTH, HEIGHT).unwrap();
            }
            None => {
                #[cfg(not(target_arch = "wasm32"))]
                {
                    let mut stdout = io::stdout();
                    let mut i = 0;
                    for char_row in 0..29 {
                        for char_col in 0..64 {
                            let ascii = self.read(0x6000 + i);
                            i += 1;
                            let char = ASCII_LOOKUP[ascii as usize];
                            let _ = write!(
                                // FG_COLOR = 0xFFCC00
                                // BG_COLOR = 0x110500
                                stdout,
                                "{}{}{}{char}",
                                Goto(char_col + 1, char_row + 1),
                                Fg(color::Rgb(0xFF, 0xCC, 0x00)),
                                Bg(color::Rgb(0x11, 0x05, 0x00))
                            );
                        }
                    }
                }
            }
        }
        self.controller.irq();
    }
}

impl MemoryReader for Renderer {
    fn read(&self, address: u16) -> u8 {
        self.memory.read(address)
    }
}
